
/********************************************************/
/* effect processor orchestra with chorus, delay,	*/
/* reverb and spatialization effects	v 1.2		*/
/* written by Istvan Varga, Nov 2001			*/
/********************************************************/

sr	=  __SRATE
ksmps	=  __KSMPS
nchnls	=  2

#include "spat.h.orc"

/* ReverbOn - output with reverb */

#define ReverbOn #

irvlen	=  (__REVERBLENGTH + 0.1) / sr	/* reverb delay */
ilnth	=  p3 + 0.12
ilnth2	=  ilnth + irvlen
p3	=  ilnth2 + irvlen		/* extend note duration */

aenv	linseg 1.0, ilnth - 0.12, 1.0, 0.1, 0.0, 1.0, 0.0	/* de-click */
aenv2	linseg 1.0, ilnth2 - 0.12, 1.0, 0.1, 0.0, 1.0, 0.0

irvvol	table  0, 90			/* reverb volume */

axL	delay aL * aenv2, irvlen	/* sync. output with reverb */
axR	delay aR * aenv2, irvlen

aL, aR	convolve a0 * aenv * irvvol, __RVB_FILE		/* reverberate input */

aL	=  aL + axL				/* mix to output */
aR	=  aR + axR

#

/* ReverbOff - output without reverb */

#define ReverbOff #

ilnth	=  p3 + 0.12
p3	=  ilnth			/* extend note duration */

aenv	linseg 1.0, ilnth - 0.12, 1.0, 0.1, 0.0, 1.0, 0.0	/* de-click */

aL	=  aL * aenv
aR	=  aR * aenv

#

#include __CONF_NAME

gax0	init 0
gax0L	init 0
gax0R	init 0
garvb	init 0

/* ------ create default ftables ------ */

i_	ftgen 101, 0, 257, -7, 0.0, 257, 0.0		/* azimuth control   */
i_	ftgen 102, 0, 257, -7, 0.0, 257, 0.0		/* elevation control */
i_	ftgen 103, 0, 257, -7, 1.0, 257, 1.0		/* distance control  */

i_	ftgen   1, 0,  32, -2, 1.0, 1000.0, 1.0, 0.7071, 0,		\
		1000.0, 1.0, 0.7071, 0, 1000.0, 1.0, 0.7071, 0,		\
		1000.0, 1.0, 0.7071, 0		/* global scale and EQ	*/

i_	ftgen  21, 0,  32, -2, 1, 1.0, 101, 5.0, 1.0, 1, 102, 5.0, 1.0, 1,  \
		103, 5.0, 1.0, 1, 0.0, 0.0, 1.0	/* main channel		*/

i_	ftgen  22, 0,   4, -2, 0, 1.0		/* stereo input		*/

i_	ftgen   2, 0,  32, -2, 0, 1.0		/* channel 1		*/
i_	ftgen   3, 0,  32, -2, 0, 1.0		/* channel 2		*/
i_	ftgen   4, 0,  32, -2, 0, 1.0		/* channel 3		*/
i_	ftgen   5, 0,  32, -2, 0, 1.0		/* channel 4		*/
i_	ftgen   6, 0,  32, -2, 0, 1.0		/* channel 5		*/
i_	ftgen   7, 0,  32, -2, 0, 1.0		/* channel 6		*/
i_	ftgen   8, 0,  32, -2, 0, 1.0		/* channel 7		*/
i_	ftgen   9, 0,  32, -2, 0, 1.0		/* channel 8		*/

i_	ftgen  90, 0,   4, -2, 1.0		/* reverb level		*/

i_	ftgen  10, 0, 4096, 10, 1.0		/* sine and triangle wave */
i_	ftgen  11, 0, 4096,  7, 0.0, 1024, 1.0, 2048, -1.0, 1024, 0.0

	instr 1		/* mono input and scheduler instrument */

/* initialization */

	kgoto start01

iflen	filelen __INFILENAME	/* set p3 according to input file length */
p3	=  iflen + p4 + 1.0 / kr

ichn	=  1				/* initialize delay channels	*/
chn_init1:
ifno	=  int(ichn + 1.5)		/* table number			*/
insno	=  (ichn < 4.5 ? 2 : 3)		/* instrument number		*/
itmp	table 0, ifno
	if (itmp < 0.5) igoto chn_init2		/* channel disbled	*/
	schedkwhen 1, -1, -1, insno, 0, p3, ifno
chn_init2:
ichn	=  ichn + 1
	if (ichn < 8.5) igoto chn_init1		/* channels 1 - 8	*/

itmp	table 0, 21				/* main channel		*/
	if (itmp < 0.5) igoto chn_init3
	schedkwhen 1, -1, -1, 21, 0, p3
chn_init3:

itmp	table 0, 22				/* stereo input		*/
	if (itmp < 0.5) igoto chn_init4
	schedkwhen 1, -1, -1, 22, 0, p3
chn_init4:

	schedkwhen 1, -1, -1, 90, 0, p3		/* reverb / output	*/

/* performance */

start01:

a0	soundin __INFILENAME			/* read input file	*/

i01	=  1.0 / 1.0e5			/* noise generator */
i02	=  1.0 / 1.0e19
a_	phasor sr * 0.419417
a_	=  a_ * a_
a1	wrap a_ * 1.08041, -i01, i01
a2	wrap a_ * 1.59911, -i01, i01
a1	=  a1 * i02
gax0	=  a1 + a0			/* avoid underflow */
gax0L	=  a1
gax0R	=  a1
garvb	=  a2 * i02

	endin

/* ------ main (non-delayed) channel ------ */

	instr 21

p3	=  p3 + 0.1		/* increase note length */

ivolx	table 1,	21	/* volume */

iaztn	table 2,	21	/* azimuth table number */
iaztl	table 3,	21	/* azimuth table length in seconds */
iazts	table 4,	21	/* azimuth table scale value */
iaztm	table 5,	21	/* wrap mode */

ielvn	table 6,	21	/* elevation table number */
ielvl	table 7,	21	/* elevation table length in seconds */
ielvs	table 8,	21	/* elevation table scale value */
ielvm	table 9,	21	/* wrap mode */

idstn	table 10,	21	/* distance table number */
idstl	table 11,	21	/* distance table length in seconds */
idsts	table 12,	21	/* distance table scale value */
idstm	table 13,	21	/* wrap mode */

i_az_	table 14,	21	/* azimuth */
i_el_	table 15,	21	/* elevation */
i_d_	table 16,	21	/* distance */

/* performance */

ktpos	times			/* read controllers from function tables */

k_az_	table3 ktpos / iaztl, iaztn, 1, 0, iaztm
k_el_	table3 ktpos / ielvl, ielvn, 1, 0, ielvm
k_d_	table3 ktpos / idstl, idstn, 1, 0, idstm

k_az_	=  k_az_ * iazts		/* scale values from table */
k_el_	=  k_el_ * ielvs
k_d_	=  k_d_ * idsts

i_0_	table 0, iaztn		/* init. values for controllers */
i_1_	table 0, ielvn
i_2_	table 0, idstn

k_az_	init i_0_ * iazts
k_el_	init i_1_ * ielvs
k_d_	init i_2_ * idsts

a_	=  gax0 * ivolx	/* get input signal */

garvb	=  garvb + a_		/* send reverb */

$SPATIALIZE1			/* convert to stereo and send to output */

gax0L	=  gax0L + a_L_
gax0R	=  gax0R + a_R_

	endin			/* end of main instrument */

/* ------ mix channel / stereo input file ------ */

	instr 22

	kgoto start01

p3	=  p3 + 0.1		/* increase note length */

ivol	table 1, 22		/* volume */

/* performance */

start01:

aL, aR	soundin __INFIL2NAME	/* read input file */

gax0L	=  gax0L + aL * ivol	/* send to output */
gax0R	=  gax0R + aR * ivol

a0	=  gax0			/* send reverb */
garvb	=  garvb + a0 * ivol

	endin

/* ------ instr 2: delay channel with chorus effect ------ */

	instr 2

/* initialization */

p3	=  p3 + 1.0			/* increase note length	*/

ivol	table  1, p4		/* volume		*/

iEQ1f	table  2, p4		/* EQ 1 frequency	*/
iEQ1l	table  3, p4		/* EQ 1 level		*/
iEQ1Q	table  4, p4		/* EQ 1 Q		*/
iEQ1m	table  5, p4		/* EQ 1 mode		*/

iEQ2f	table  6, p4		/* EQ 2 frequency	*/
iEQ2l	table  7, p4		/* EQ 2 level		*/
iEQ2Q	table  8, p4		/* EQ 2 Q		*/
iEQ2m	table  9, p4		/* EQ 2 mode		*/

iEQ3f	table 10, p4		/* EQ 3 frequency	*/
iEQ3l	table 11, p4		/* EQ 3 level		*/
iEQ3Q	table 12, p4		/* EQ 3 Q		*/
iEQ3m	table 13, p4		/* EQ 3 mode		*/

i_az_	table 14, p4		/* azimuth		*/
i_el_	table 15, p4		/* elevation		*/
i_d_	table 16, p4		/* distance		*/

idel0	table 17, p4		/* delay offset		*/

idel	table 18, p4		/* delay time		*/
ifblvl	table 19, p4		/* feedback level	*/

idmdep	table 20, p4		/* delay modulation depth	*/
idmfrq	table 21, p4		/* delay modulation frequency	*/
idmphs	table 22, p4		/* LFO start phase (0..1)	*/
idmwfm	table 23, p4		/* LFO waveform			*/

ifbEQf	table 24, p4		/* feedback EQ frequency	*/
ifbEQl	table 25, p4		/* feedback EQ level		*/
ifbEQQ	table 26, p4		/* feedback EQ Q		*/
ifbEQm	table 27, p4		/* feedback EQ mode		*/

/* performance */

a0	=  gax0 * ivol			/* get input signal	*/

a0	pareq a0, iEQ1f, iEQ1l, iEQ1Q, iEQ1m	/* equalizer	*/
a0	pareq a0, iEQ2f, iEQ2l, iEQ2Q, iEQ2m
a0	pareq a0, iEQ3f, iEQ3l, iEQ3Q, iEQ3m

aLFOsig	oscili 1.0, idmfrq, int(idmwfm + 10.5), idmphs	/* LFO	*/

a1	delay a0, idel0			/* delay offset		*/

a_	delayr idel + idmdep + (ksmps + 14) / sr
adel	=  idel + aLFOsig * idmdep
a_	deltapx adel, 24
afb	pareq a_, ifbEQf, ifbEQl, ifbEQQ, ifbEQm	/* feedback EQ	     */
	delayw a1 + afb * ifblvl			/* delay w/ feedback */

garvb	=  garvb + a_			/* reverb		*/

$SPATIALIZE2

gax0L	=  gax0L + a_L_			/* mix to output	*/
gax0R	=  gax0R + a_R_

	endin


/* ------ instr 3: delay channel without chorus effect ------ */

	instr 3

/* initialization */

p3	=  p3 + 1.0			/* increase note length	*/

ivol	table  1, p4		/* volume	*/

iEQ1f	table  2, p4		/* EQ 1 frequency	*/
iEQ1l	table  3, p4		/* EQ 1 level		*/
iEQ1Q	table  4, p4		/* EQ 1 Q		*/
iEQ1m	table  5, p4		/* EQ 1 mode		*/

iEQ2f	table  6, p4		/* EQ 2 frequency	*/
iEQ2l	table  7, p4		/* EQ 2 level		*/
iEQ2Q	table  8, p4		/* EQ 2 Q		*/
iEQ2m	table  9, p4		/* EQ 2 mode		*/

iEQ3f	table 10, p4		/* EQ 3 frequency	*/
iEQ3l	table 11, p4		/* EQ 3 level		*/
iEQ3Q	table 12, p4		/* EQ 3 Q		*/
iEQ3m	table 13, p4		/* EQ 3 mode		*/

i_az_	table 14, p4		/* azimuth		*/
i_el_	table 15, p4		/* elevation		*/
i_d_	table 16, p4		/* distance		*/

idel	table 17, p4		/* delay time		*/

ifbdel	table 18, p4		/* feedback delay		*/
ifblvl	table 19, p4		/* feedback level		*/
ifbEQf	table 20, p4		/* feedback EQ frequency	*/
ifbEQl	table 21, p4		/* feedback EQ level		*/
ifbEQQ	table 22, p4		/* feedback EQ Q		*/
ifbEQm	table 23, p4		/* feedback EQ mode		*/

/* performance */

a0	=  gax0 * ivol			/* get input signal */

a0	pareq a0, iEQ1f, iEQ1l, iEQ1Q, iEQ1m	/* equalizer */
a0	pareq a0, iEQ2f, iEQ2l, iEQ2Q, iEQ2m
a0	pareq a0, iEQ3f, iEQ3l, iEQ3Q, iEQ3m

a1	delay a0, idel				/* delay		*/

a_	delayr ifbdel				/* feedback with EQ	*/
a_	=  a1 + a_
afb	pareq ifblvl * a_, ifbEQf, ifbEQl, ifbEQQ, ifbEQm
	delayw afb

garvb	=  garvb + a_				/* reverb	*/

$SPATIALIZE2

gax0L	=  gax0L + a_L_				/* mix to output */
gax0R	=  gax0R + a_R_

	endin

/* ------ instr 90: reverb / output instrument ------ */

	instr 90

	kgoto start01

ivol	table  0, 1	/* volume		*/

iEQ1f	table  1, 1	/* EQ 1 frequency	*/
iEQ1l	table  2, 1	/* EQ 1 level		*/
iEQ1Q	table  3, 1	/* EQ 1 Q		*/
iEQ1m	table  4, 1	/* EQ 1 mode		*/

iEQ2f	table  5, 1	/* EQ 2 frequency	*/
iEQ2l	table  6, 1	/* EQ 2 level		*/
iEQ2Q	table  7, 1	/* EQ 2 Q		*/
iEQ2m	table  8, 1	/* EQ 2 mode		*/

iEQ3f	table  9, 1	/* EQ 3 frequency	*/
iEQ3l	table 10, 1	/* EQ 3 level		*/
iEQ3Q	table 11, 1	/* EQ 3 Q		*/
iEQ3m	table 12, 1	/* EQ 3 mode		*/

iEQ4f	table 13, 1	/* EQ 4 frequency	*/
iEQ4l	table 14, 1	/* EQ 4 level		*/
iEQ4Q	table 15, 1	/* EQ 4 Q		*/
iEQ4m	table 16, 1	/* EQ 4 mode		*/

/* performance */

start01:

a0	=  garvb	/* get input */
aL	=  gax0L
aR	=  gax0R

gax0	=  0		/* clear global variables */
gax0L	=  0
gax0R	=  0
garvb	=  0

$REVERB			/* reverberate */

aL	pareq aL, iEQ1f, iEQ1l, iEQ1Q, iEQ1m	/* apply EQ */
aL	pareq aL, iEQ2f, iEQ2l, iEQ2Q, iEQ2m
aL	pareq aL, iEQ3f, iEQ3l, iEQ3Q, iEQ3m
aL	pareq aL, iEQ4f, iEQ4l, iEQ4Q, iEQ4m

aR	pareq aR, iEQ1f, iEQ1l, iEQ1Q, iEQ1m
aR	pareq aR, iEQ2f, iEQ2l, iEQ2Q, iEQ2m
aR	pareq aR, iEQ3f, iEQ3l, iEQ3Q, iEQ3m
aR	pareq aR, iEQ4f, iEQ4l, iEQ4Q, iEQ4m

	outs aL * ivol, aR * ivol			/* write output */

	endin

