
/* test convolve file and print delay in samples */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

/* convolve header structure (float version) */

typedef struct {
	long		cvmagic;	/* magic number (666)		*/
	long		hdrsize;	/* header size in bytes		*/
	long		datasize;	/* data size in bytes		*/
	long		datafmt;	/* data format			*/
	float		srate;		/* sample rate			*/
	long		nchnls;		/* number of channels in source	*/
	long		channel;	/* requested channel		*/
	long		irsize;		/* IR length in samples		*/
	long		format;		/* format specifier (1L)	*/
	char		info[4];	/* not used			*/
} cvhead_float;

/* convolve header structure (double version) */

typedef struct {
	long		cvmagic;	/* magic number (666)		*/
	long		hdrsize;	/* header size in bytes		*/
	long		datasize;	/* data size in bytes		*/
	long		datafmt;	/* data format			*/
	double		srate;		/* sample rate			*/
	long		nchnls;		/* number of channels in source	*/
	long		channel;	/* requested channel		*/
	long		irsize;		/* IR length in samples		*/
	long		format;		/* format specifier (1L)	*/
	char		info[4];	/* not used			*/
} cvhead_double;

int	main (int argc, char **argv)
{
	FILE		*f;
	long		l, ksmps, fsize;
	double		sr, esr;
	cvhead_float	hdr4;
	cvhead_double	hdr8;

	if (argc != 4) {
		fprintf (stderr, "TestCvFile2 <filename> <srate> <ksmps>\n");
		exit (-1);
	}

	f = fopen (argv[1], "rb");	/* file name 	*/
	esr = (double) atof (argv[2]);	/* sample rate	*/
	ksmps = (long) atoi (argv[3]);	/* ksmps	*/

	if (f == NULL) exit (-1);		/* file does not exist	*/
	fseek (f, 0, SEEK_END);
	fsize = (long) ftell (f);
	if (fsize < (long) sizeof (cvhead_double))
		exit (-1);				/* file is too short */
	fseek (f, 0, SEEK_SET);
	fread (&hdr4, sizeof (cvhead_float), 1, f);
	fseek (f, 0, SEEK_SET);
	fread (&hdr8, sizeof (cvhead_double), 1, f);
	fclose (f);

	if (hdr4.hdrsize == sizeof (cvhead_float)) {
		if (hdr4.cvmagic != 666L) exit (-1);	/* not a .cv file */
		if (hdr4.nchnls != 2L) exit (-1);	/* file is not stereo */
		sr = (double) hdr4.srate;
		l = hdr4.irsize;
	} else if (hdr8.hdrsize == sizeof (cvhead_double)) {
		if (hdr8.cvmagic != 666L) exit (-1);	/* not a .cv file */
		if (hdr8.nchnls != 2L) exit (-1);	/* file is not stereo */
		sr = hdr8.srate;
		l = hdr8.irsize;
	} else {
		exit (-1);		/* invalid header length */
	}

	if ((l <= 0L) || (ksmps <= 0L)) exit (-1);

	if (sr == 0.0) exit (-1);	/* invalid sample rate */
	if (fabs ((esr / sr) - 1.0) > 5.0e-7) exit (-1);

	/* calculate and print delay */

	if (l >= ksmps) {
		l = ((l + ksmps - 1L) / ksmps) * ksmps;
	} else {
		l = ((ksmps + l - 1L) / l) * l;
	}
	printf (" %ld ", l);

	return 0;
}

